@@ -1,19 +1,18 @@
 /*
  * Copyright (C) 2020 Alexander Mikhaylenko <alexm@gnome.org>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #pragma once
 
 #include <gladeui/glade.h>
 
-#define HANDY_USE_UNSTABLE_API
 #include <handy.h>
 
 #define ONLY_THIS_GOES_IN_THAT_MSG _("Only objects of type %s can be added to objects of type %s.")
 
-/* Guess wether we are using a Glade version older than 3.36.
+/* Guess whether we are using a Glade version older than 3.36.
  *
  * If yes, redefine some symbols which got renamed.
  */
@@ -22,6 +21,16 @@
 #define glade_widget_action_get_def glade_widget_action_get_class
 #endif
 
+/*
+ * Another macro got renamed in GladeUI 3.38. For older versions of GladeUI,
+ * define the new macro as the old one.
+ */
+#ifndef GLADE_WIDGET_ADAPTOR_GET_ADAPTOR_CLASS
+#define GLADE_WIDGET_ADAPTOR_GET_ADAPTOR_CLASS GWA_GET_CLASS
+#endif
+
+void glade_hdy_init (const gchar *name);
+
 void glade_hdy_sync_child_positions (GtkContainer *container);
 
 gint glade_hdy_get_child_index (GtkContainer *container,
