@@ -1,23 +1,24 @@
 /*
  * Copyright (C) 2019 Alexander Mikhaylenko <exalm7659@gmail.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
+#include "config.h"
+
 #include "hdy-swipeable.h"
 
 /**
- * SECTION:hdy-swipeable
- * @short_description: An interface for swipeable widgets.
- * @title: HdySwipeable
- * @See_also: #HdyCarousel, #HdyDeck, #HdyLeaflet, #HdySwipeGroup
+ * HdySwipeable:
  *
- * The #HdySwipeable interface is implemented by all swipeable widgets. They
- * can be synced using #HdySwipeGroup.
+ * An interface for swipeable widgets.
  *
- * See #HdySwipeTracker for details about implementing it.
+ * The `HdySwipeable` interface is implemented by all swipeable widgets. They
+ * can be synced using [class@SwipeGroup].
  *
- * Since: 0.0.12
+ * See [class@SwipeTracker] for details about implementing it.
+ *
+ * Since: 1.0
  */
 
 G_DEFINE_INTERFACE (HdySwipeable, hdy_swipeable, GTK_TYPE_WIDGET)
@@ -34,15 +35,15 @@ hdy_swipeable_default_init (HdySwipeableInterface *iface)
 {
   /**
    * HdySwipeable::child-switched:
-   * @self: The #HdySwipeable instance
+   * @self: a swipeable
    * @index: the index of the child to switch to
-   * @duration: Animation duration in milliseconds
+   * @duration: animation duration, in milliseconds
    *
-   * This signal should be emitted when the widget's visible child is changed.
+   * Emitted when the widget's visible child is changed.
    *
    * @duration can be 0 if the child is switched without animation.
    *
-   * This is used by #HdySwipeGroup, applications should not connect to it.
+   * This is used by [class@SwipeGroup], applications should not connect to it.
    *
    * Since: 1.0
    */
@@ -59,11 +60,13 @@ hdy_swipeable_default_init (HdySwipeableInterface *iface)
 
 /**
  * hdy_swipeable_switch_child:
- * @self: a #HdySwipeable
+ * @self: a swipeable
  * @index: the index of the child to switch to
- * @duration: Animation duration in milliseconds
+ * @duration: animation duration, in milliseconds
  *
- * See HdySwipeable::child-switched.
+ * Switches to child with index @index.
+ *
+ * See [signal@Swipeable::child-switched].
  *
  * Since: 1.0
  */
@@ -79,17 +82,18 @@ hdy_swipeable_switch_child (HdySwipeable *self,
   iface = HDY_SWIPEABLE_GET_IFACE (self);
   g_return_if_fail (iface->switch_child != NULL);
 
-  (* iface->switch_child) (self, index, duration);
+  iface->switch_child (self, index, duration);
 }
 
 /**
  * hdy_swipeable_emit_child_switched:
- * @self: a #HdySwipeable
+ * @self: a swipeable
  * @index: the index of the child to switch to
- * @duration: Animation duration in milliseconds
+ * @duration: animation duration, in milliseconds
  *
- * Emits HdySwipeable::child-switched signal. This should be called when the
- * widget switches visible child widget.
+ * Emits [signal@Swipeable::child-switched] signal.
+ *
+ * This should be called when the widget switches visible child widget.
  *
  * @duration can be 0 if the child is switched without animation.
  *
@@ -107,9 +111,9 @@ hdy_swipeable_emit_child_switched (HdySwipeable *self,
 
 /**
  * hdy_swipeable_get_swipe_tracker:
- * @self: a #HdySwipeable
+ * @self: a swipeable
  *
- * Gets the #HdySwipeTracker used by this swipeable widget.
+ * Gets the [class@SwipeTracker] used by this swipeable widget.
  *
  * Returns: (transfer none): the swipe tracker
  *
@@ -125,15 +129,16 @@ hdy_swipeable_get_swipe_tracker (HdySwipeable *self)
   iface = HDY_SWIPEABLE_GET_IFACE (self);
   g_return_val_if_fail (iface->get_swipe_tracker != NULL, NULL);
 
-  return (* iface->get_swipe_tracker) (self);
+  return iface->get_swipe_tracker (self);
 }
 
 /**
  * hdy_swipeable_get_distance:
- * @self: a #HdySwipeable
+ * @self: a swipeable
  *
- * Gets the swipe distance of @self. This corresponds to how many pixels
- * 1 unit represents.
+ * Gets the swipe distance of @self.
+ *
+ * This corresponds to how many pixels 1 unit represents.
  *
  * Returns: the swipe distance in pixels
  *
@@ -149,19 +154,20 @@ hdy_swipeable_get_distance (HdySwipeable *self)
   iface = HDY_SWIPEABLE_GET_IFACE (self);
   g_return_val_if_fail (iface->get_distance != NULL, 0);
 
-  return (* iface->get_distance) (self);
+  return iface->get_distance (self);
 }
 
 /**
  * hdy_swipeable_get_snap_points: (virtual get_snap_points)
- * @self: a #HdySwipeable
+ * @self: a swipeable
  * @n_snap_points: (out): location to return the number of the snap points
  *
- * Gets the snap points of @self. Each snap point represents a progress value
- * that is considered acceptable to end the swipe on.
+ * Gets the snap points of @self.
  *
- * Returns: (array length=n_snap_points) (transfer full): the snap points of
- *     @self. The array must be freed with g_free().
+ * Each snap point represents a progress value that is considered acceptable to
+ * end the swipe on.
+ *
+ * Returns: (array length=n_snap_points) (transfer full): the snap points
  *
  * Since: 1.0
  */
@@ -176,14 +182,14 @@ hdy_swipeable_get_snap_points (HdySwipeable *self,
   iface = HDY_SWIPEABLE_GET_IFACE (self);
   g_return_val_if_fail (iface->get_snap_points != NULL, NULL);
 
-  return (* iface->get_snap_points) (self, n_snap_points);
+  return iface->get_snap_points (self, n_snap_points);
 }
 
 /**
  * hdy_swipeable_get_progress:
- * @self: a #HdySwipeable
+ * @self: a swipeable
  *
- * Gets the current progress of @self
+ * Gets the current progress of @self.
  *
  * Returns: the current progress, unitless
  *
@@ -199,12 +205,12 @@ hdy_swipeable_get_progress (HdySwipeable *self)
   iface = HDY_SWIPEABLE_GET_IFACE (self);
   g_return_val_if_fail (iface->get_progress != NULL, 0);
 
-  return (* iface->get_progress) (self);
+  return iface->get_progress (self);
 }
 
 /**
  * hdy_swipeable_get_cancel_progress:
- * @self: a #HdySwipeable
+ * @self: a swipeable
  *
  * Gets the progress @self will snap back to after the gesture is canceled.
  *
@@ -222,20 +228,25 @@ hdy_swipeable_get_cancel_progress (HdySwipeable *self)
   iface = HDY_SWIPEABLE_GET_IFACE (self);
   g_return_val_if_fail (iface->get_cancel_progress != NULL, 0);
 
-  return (* iface->get_cancel_progress) (self);
+  return iface->get_cancel_progress (self);
 }
 
 /**
  * hdy_swipeable_get_swipe_area:
- * @self: a #HdySwipeable
- * @rect: (out): a pointer to a #GdkRectangle to store the swipe area
+ * @self: a swipeable
+ * @navigation_direction: the direction of the swipe
+ * @is_drag: whether the swipe is caused by a dragging gesture
+ * @rect: (out): a pointer to a rectangle to store the swipe area
  *
- * Gets the area @self can start a swipe from. This can be used to restrict
- * swipes to only be possible from a certain area, for example, to only allow
- * edge swipes, or to have a draggable element and ignore swipes elsewhere.
+ * Gets the area @self can start a swipe from for the given direction and
+ * gesture type.
+ *
+ * This can be used to restrict swipes to only be possible from a certain area,
+ * for example, to only allow edge swipes, or to have a draggable element and
+ * ignore swipes elsewhere.
  *
  * Swipe area is only considered for direct swipes (as in, not initiated by
- * #HdySwipeGroup).
+ * [class@SwipeGroup]).
  *
  * If not implemented, the default implementation returns the allocation of
  * @self, allowing swipes from anywhere.
@@ -243,8 +254,10 @@ hdy_swipeable_get_cancel_progress (HdySwipeable *self)
  * Since: 1.0
  */
 void
-hdy_swipeable_get_swipe_area (HdySwipeable *self,
-                              GdkRectangle *rect)
+hdy_swipeable_get_swipe_area (HdySwipeable           *self,
+                              HdyNavigationDirection  navigation_direction,
+                              gboolean                is_drag,
+                              GdkRectangle           *rect)
 {
   HdySwipeableInterface *iface;
 
@@ -254,7 +267,7 @@ hdy_swipeable_get_swipe_area (HdySwipeable *self,
   iface = HDY_SWIPEABLE_GET_IFACE (self);
 
   if (iface->get_swipe_area) {
-    (* iface->get_swipe_area) (self, rect);
+    iface->get_swipe_area (self, navigation_direction, is_drag, rect);
     return;
   }
 
