@@ -1,12 +1,13 @@
 /*
  * Copyright (C) 2019 Alexander Mikhaylenko <exalm7659@gmail.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
-#define HANDY_USE_UNSTABLE_API
 #include <handy.h>
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static void
 test_hdy_swipe_group_add_remove (void)
 {
@@ -16,8 +17,8 @@ test_hdy_swipe_group_add_remove (void)
 
   group = hdy_swipe_group_new ();
 
-  swipeable1 = HDY_SWIPEABLE (hdy_carousel_new ());
-  swipeable2 = HDY_SWIPEABLE (hdy_carousel_new ());
+  swipeable1 = g_object_ref_sink (HDY_SWIPEABLE (hdy_carousel_new ()));
+  swipeable2 = g_object_ref_sink (HDY_SWIPEABLE (hdy_carousel_new ()));
 
   g_assert_cmpint (g_slist_length (hdy_swipe_group_get_swipeables (group)), ==, 0);
 
@@ -34,11 +35,14 @@ test_hdy_swipe_group_add_remove (void)
   g_assert_cmpint (g_slist_length (hdy_swipe_group_get_swipeables (group)), ==, 0);
 }
 
+G_GNUC_END_IGNORE_DEPRECATIONS
+
 gint
 main (gint argc,
       gchar *argv[])
 {
   gtk_test_init (&argc, &argv, NULL);
+  hdy_init ();
 
   g_test_add_func("/Handy/SwipeGroup/add_remove", test_hdy_swipe_group_add_remove);
   return g_test_run();
